<?php defined('SYSPATH') or die('No direct script access.');
class Auth_Controller extends Dashboard_Controller {
	public function index()
	{
		// have a request or go home
		url::redirect('home');
	}

	public function register() {
		if($post = $this->input->post()) {
			$post = new Validation($post);
			$post->add_rules('username','required','valid::alpha_dash','length[3,30]');
			$post->add_rules('password','required','length[3,30]','matches[conf_password]');
			$post->add_rules('email','required','valid::email_rfc');
			if(isset($post['create_net']) && $post['create_net']=='create_net') {
				$post->add_rules('location','required','valid::standard_text');
			}
			$this->username_exists($post,'username');
			if($post->validate()) {
				// no validation errors found, add user to db and log in
				$user = ORM::factory('user');
				$user->add(ORM::factory('role', 'login'));
				$user->username = $post['username'];
				$user->email = $post['email'];
				$user->password = $post['password'];
				$user->language = isset($post['language']) ? $post['language'] : "en_US";

				if(isset($post['create_net']) && $post['create_net']=='create_net') {
					$network = ORM::factory('network');
					$network->name = $post['username'];
					$network->display_name = 'The '.ucwords(str_replace("_"," ",$post['username'])).' Network';
					$network->location = $post['location'];
					$network->ap1_essid = $post['username'];
					$network->ap2_essid = $post['username']."_private";
					$network->ap2_key = "0p3nm35h";
					$network->save();
					$user->add($network);
				}
				$user->save();
				Auth::instance()->login($user->username, $post['password']);
				url::redirect('home');
			} else {
				$errors = $post->errors('form_errors');
				$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.register');
				$this->template->content = new View('user/register');
				foreach($errors as $key => $value) {
					$this->session->set_flash(array('alert'=>$value,'type'=>'bad'));
				}
			}
		} else {
			$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.register');
			$this->template->content = new View('user/register');
		}
	}

	public function login($role="")
	{
		if (Auth::instance()->logged_in($role))
		{
			if(isset($this->session)) {
				url::redirect($this->session->get("requested_url")); //return to page where login was called
			} else {
				url::redirect('home');
			}
		}
		else
		{
			if (Auth::instance()->logged_in()){
				$this->template->title="No Access";
				$this->template->content=new View('pages/noaccess');
			}else{
				$this->template->title=Kohana::lang('base.dashname')." | ".Kohana::lang('base.login');
				$this->template->content= new View('user/login');
			}
		}
		$form = $this->input->post();
		if($form)
		{
			// Load the user
			$user = ORM::factory('user', $form['username']);
			echo "User: " . $user;
			if($user->password != ""){
				// orm user object or $form['username'] could be used
				if(!Auth::instance()->login($form['username'], $form['password'])) {
					$this->session->set('login_error','true');
				}
			} else {
				$this->session->set('login_error','true');
			}
			url::redirect('auth/login');
		}
	}

	public function logout() {
		Auth::instance()->logout(true);
		url::redirect('home');
	}


	public static function username_exists(Validation $array, $field ) {
		// check the database for existing records
		$name_exists = (bool) ORM::factory('user')->where('username',$array[$field])->count_all();

		if ($name_exists)
		{
			// add error to validation object
			$array->add_error($field, 'username_exists');
		}

		return $name_exists;
	}
}
