<?php defined('SYSPATH') or die('No direct script access.');
class Network_Controller extends Protected_Controller {
	function __construct() {
		parent::__construct();
	}


	function index() {
		$this->auto_render = FALSE;
	}

	/*
	 * Display proper network page options for login.
	 */
	function overview($net_name = "") {
		// Check if the authenticated user is a member of the requested network
		// If they are, but are not editing session network, switch session network.
		$network = ORM::factory('network')->where('name',$net_name)->find();
		if($network->is_user_member(Auth::instance()->get_user()->username)){
			if($net_name != $this->session->get('current_network')) {
				$this->session->set('current_network',$net_name);
			}
			$this->is_member = true;
		} else {
			if($net_name != $this->session->get('current_network')) {
				$this->session->set('current_network',$net_name);
			}

			$this->is_member = false;
		}

		$this->get_network($net_name);
		$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.network_overview');
		$this->template->content = new View('network/overview');
	}

	/**
	 * Load the status view for the network.
	 *
	 * Loads the view given by 'network.view' variable, if set. Otherwise, loads the netlist.
	 *
	 *
	 * @param $net_name
	 * @return unknown_type
	 */
	function status($net_name = "") {
		$this->net_name = $net_name;

		$network = ORM::factory('network')->where('name',$net_name)->find();

		$this->network_stats['name'] = $network->name;
		$this->network_stats['location'] = $network->location;
		$this->network_stats['num_users'] = 0;
		$this->network_stats['total_nodes'] = 0;
		$this->network_stats['down_nodes'] = 0;
		$this->network_stats['kbup'] = 0;
		$this->network_stats['kbdown'] = 0;

		$i=0;
		foreach($network->nodes as $node) {
			if(!$node->is_fine()) {
				$this->network_stats['down_nodes'] += 1;
			}
			$this->network_stats['total_nodes'] += 1;
			$this->network_stats['kbup'] += $node->kbup;
			$this->network_stats['kbdown'] += $node->kbdown;
			$this->network_stats['num_users'] += $node->users;

			$this->node_list[$i]['name'] = $node->name;
			$this->node_list[$i]['network'] = $network->name;
			$this->node_list[$i]['time'] = $node->time;
			$this->node_list[$i]['users'] = $node->users;
			$this->node_list[$i]['kbup'] = $node->kbup;
			$this->node_list[$i]['kbdown'] = $node->kbdown;
			$this->node_list[$i]['latitude'] = $node->latitude;
			$this->node_list[$i]['longitude'] = $node->longitude;
			$this->node_list[$i]['gateway'] = $node->gateway;

			$i++;
		}

		if(Kohana::config('network.view')){
			$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.view_network');
			$this->template->content = new View(Kohana::config('network.view'));
		} else {
			$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.view_network');
			$this->template->content = new View('network/netlist');
		}
	}

	public function edit($net_name = null) {
		// Check if the authenticated user is a member of the requested network
		// If they are, but are not editing session network, switch session network.
		$network = ORM::factory('network')->where('name',$net_name)->find();
		if($network->is_user_member(Auth::instance()->get_user()->username)){
			if($net_name != $this->session->get('current_network')) {
				$this->session->set('current_network',$net_name);
			}
		} else {
			$this->error = true;
			$this->session->set_flash(array('alert'=>Kohana::lang('base.no_access_network'),'type'=>'bad'));
			url::redirect('home');
		}

		// Handle form input
		if(isset($_POST['type'])) {
			switch($this->input->post('type')) {
				case 'general':
					$this->procEditGeneral($net_name);
					break;
				case 'ap1':
					$this->procEditPublic($net_name);
					break;
				case 'ap2':
					$this->procEditPrivate($net_name);
					break;
				case 'captive':
					$this->procEditCaptive($net_name);
					break;
				case 'advanced':
					$this->procEditAdvanced($net_name);
					break;
				default:
					$this->index();
					break;
			}
		}

		// Display form if not processing submission
		$this->get_network($net_name);
		$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.edit_network');
		$this->template->content = new View('network/edit');
	}

	/*
	 * Process the submission of the general settings form
	 */
	private function procEditGeneral($net_name) {
		$post = new Validation($_POST);
		$post->add_rules('name','valid::alpha_numeric','length[0,30]');
		$post->add_rules('display_name','valid::standard_text','length[0,200]');
		$post->add_rules('location','valid::alpha_numeric','length[0,100]');

		if($post->validate()){
			$name = $this->input->post('name');
			$display_name = $this->input->post('display_name');
			$location = $this->input->post('location');
			$network = ORM::factory('network')->where('name',$net_name)->find();
			if(isset($network)){
				$network->name = $name;
				$network->display_name = $display_name;
				$network->location = $location;
				$network->save();
				$this->session->set_flash(array('alert'=>Kohana::lang('base.settings_success'), 'type'=>'good'));
			} else {
				$this->session->set_flash(array('alert'=>Kohana::lang('base.settings_fail'),'type'=>'bad'));
			}
		}

		url::redirect('network/edit/'.$net_name);


	}

	private function procEditPublic() {
		$post = new Validation($_POST);
		$post->add_rules('ap1_essid','valid::alpha_numeric','length[0,30]');
		$post->add_rules('ap1_key','valid::standard_text','length[0,200]');

		if($post->validate()){
			$name = $this->input->post('name');
			$display_name = $this->input->post('display_name');
			$location = $this->input->post('location');
			if($network = ORM::factory('network')->where('name',$net_name)->find()){
				$network->name = $name;
				$network->display_name = $display_name;
				$network->location = $location;
				$network->save();
				$this->session->set_flash(array('alert'=>Kohana::lang('base.settings_success'), 'type'=>'good'));
			} else {
				$this->session->set_flash(array('alert'=>Kohana::lang('base.settings_fail'),'type'=>'bad'));
			}
		}

		url::redirect('network/edit/'.$net_name);

	}


	/**
	 * Show the add/edit nodes page
	 * @return unknown_type
	 */
	public function nodes($net_name = "") {

		$network = ORM::factory('network')->where('name',$net_name)->find();
		if($network->is_user_member(Auth::instance()->get_user()->username)){
			if($net_name != $this->session->get('current_network')) {
				$this->session->set('current_network',$net_name);
			}
		} else {
			$this->error = true;
			$this->session->set_flash(array('alert'=>Kohana::lang('base.no_access_network'),'type'=>'bad'));
			url::redirect('home');
		}

		if(isset($_POST['type'])) {
			switch($this->input->post('type')) {
				case 'add':
					$this->procNodesAdd();
					break;
				case 'edit':
					$this->procNodesEdit();
					break;
				default:
					$this->index();
					break;
			}
		}

		$this->get_network($net_name);
		$this->template->title = Kohana::lang('base.dashname')." | ".Kohana::lang('base.manage_nodes');
		$this->template->content = new View('network/nodes');

	}

	private function procNodesAdd() {
		$user = ORM::factory('user')->where('username',Auth::instance()->get_user()->username)->find();

		$post = new Validation($_POST);
		$post->add_rules('name','valid::alpha_numeric','length[0,30]');
		$post->add_rules('mac','required','length[12,17]');
		$post->add_rules('latitude','required','valid::numeric');
		$post->add_rules('longitude','required','valid::numeric');
		$post->add_rules('description','valid::standard_text');

		// Logged in user must be member of the network to add nodes.
		$this_user = Auth::instance()->get_user()->username;
		$network = ORM::factory('network')->where('name',$this->session->get('current_network'))->find();

		if($post->validate()){
			$name = $this->input->post('name');
			$mac = $this->input->post('mac');
			$lat = $this->input->post('latitude');
			$long = $this->input->post('longitude');
			$description = $this->input->post('description');

			if($network->has(ORM::factory('user')->where('username',$this_user)->find())){
				$node = ORM::factory('node');
				$node->mac=$mac;
				isset($name)&&$name!="" ? $node->name=$name : $node->name=$network->name.$mac;
				$node->latitude = $lat;
				$node->longitude = $long;
				$node->description = $description;
				$node->network_id = $network->id;
				$node->uptime_history = serialize(array());
				$node->save();

				$user = ORM::factory('user')->where('username',$this_user)->find();
				$node->add($user);
				$user->add($node);
				$user->save();


				$this->session->set_flash(array('alert'=>'Node added!','type'=>'good'));

			} else {
				$this->session->set_flash(array('alert'=>Kohana::lang('base.not_allowed_add_nodes'),'type'=>'bad'));
			}
		}

		else {
			$this->session->set_flash(array('alert'=>'Sorry, there was a problem with the settings you entered.','type'=>'bad'));
		}

		url::redirect('network/nodes/'.$network->name);

	}

	private function get_network($net_name) {
		if($net_name == "") {
			$this->error = true;
			$this->session->set_flash(array('alert'=>Kohana::lang('base.no_network_provided'),'type'=>'bad'));
		} else {
			$this->network = ORM::factory('network')->where('name',$net_name)->find();
		}

	}
}
